package cellmodels.simplecell;

import java.util.*;
import simulator.*;
import util.*;

class SimpleCell extends Cell
{
    Vector sensedChemicalsXplus;
    Vector sensedChemicalsXminus;
    Vector sensedChemicalsYplus;
    Vector sensedChemicalsYminus;
    Vector sensedChemicalsZplus;
    Vector sensedChemicalsZminus;
    Vector sensedChemicals;
    
    Vector chemicalsToEmit;
    Vector chemicalsToStore;
    Vector chemicalsToDiffuse;
    
    String givenDirection;
    
    SimpleCell (Integer p_ID, SystemModel p_sys, CellProgram p_cp, Vector args)
    {
	super(p_ID,p_sys,p_cp);
	sensedChemicals = new Vector();
	sensedChemicalsXplus = new Vector();
	sensedChemicalsXminus = new Vector();
	sensedChemicalsYplus = new Vector();
	sensedChemicalsYminus = new Vector();
	sensedChemicalsZplus = new Vector();
	sensedChemicalsZminus = new Vector();
	
	chemicalsToEmit = new Vector();
	chemicalsToStore = new Vector();
	chemicalsToDiffuse = new Vector();
	givenDirection = null;
    }
    
    public void init(Vector args)
    {
	if (args != null)
	    {
		setState((String)args.elementAt(0));
	    }
    }
    
    private void resolveSingleEmission (Vector toEmit, CellEmission ce, String direction)
    {
	double conc = ce.getConcentration();
	
	if((ce.getOperator() != null) && (ce.getOperator().equals("-")))
	    {
		boolean chemWasFound = false;
		
		Enumeration enum = sensedChemicals.elements();
		while(enum.hasMoreElements())
		    {
			ChemicalConcentration cc = (ChemicalConcentration)enum.nextElement();
			if(cc.getChemical().equals(ce.getChemical()))
			    {
				conc = cc.getConcentration() - ce.getConcentration();
				chemWasFound = true;
			    }
		    }
		if(!chemWasFound || (conc < 0))
		    conc = 0.0;
	    }
	
	CellEmission resolvedEmission = new CellEmission(ce.getChemical(), null, conc, direction);
	toEmit.add(resolvedEmission);
    }
    
    private void resolveEmissions()
    {
	Vector resolvedChemicalsToEmit = new Vector();
	Enumeration enum = chemicalsToEmit.elements();
	
	while(enum.hasMoreElements())
	    {
		CellEmission ce = (CellEmission)enum.nextElement();
		String emissionDir = ce.getDirection();
		
		if((emissionDir != null) && (givenDirection != null))
		    {
			if((emissionDir.equals("same")))
			    {
				emissionDir = new String(givenDirection);
				resolveSingleEmission(resolvedChemicalsToEmit,ce,emissionDir);
			    }
			else if((emissionDir.equals("opposite")))
			    {
				if(givenDirection.equals("X+"))	emissionDir = "X-";
				else if(givenDirection.equals("X-"))	emissionDir = "X+";
				else if(givenDirection.equals("Y+"))	emissionDir = "Y-";
				else if(givenDirection.equals("Y-"))	emissionDir = "Y+";
				else if(givenDirection.equals("Z+"))	emissionDir = "Z-";
				else if(givenDirection.equals("Z-"))	emissionDir = "Z+";
				resolveSingleEmission(resolvedChemicalsToEmit,ce,emissionDir);
			    }
			else if((emissionDir.equals("perpendicular")))
			    {
				if(!givenDirection.startsWith("X"))
				    {
					resolveSingleEmission(resolvedChemicalsToEmit,ce,"X+");
					resolveSingleEmission(resolvedChemicalsToEmit,ce,"X-");
				    }
				if(!givenDirection.startsWith("Y"))
				    {
					resolveSingleEmission(resolvedChemicalsToEmit,ce,"Y+");
					resolveSingleEmission(resolvedChemicalsToEmit,ce,"Y-");
				    }
				if(!givenDirection.startsWith("Z"))
				    {
					resolveSingleEmission(resolvedChemicalsToEmit,ce,"Z+");
					resolveSingleEmission(resolvedChemicalsToEmit,ce,"Z-");
				    }
			    }
			else
			    {
				resolveSingleEmission(resolvedChemicalsToEmit,ce,emissionDir);
			    }
		    }
		else
		    {
			resolveSingleEmission(resolvedChemicalsToEmit,ce,"all");
		    }
	    }
	chemicalsToEmit = resolvedChemicalsToEmit;
	givenDirection = null;
	
	Vector argVector = new Vector();
	argVector.add("State Emissions");
	argVector.add(getState());
	Vector retVec = getCellProgram().provideService(argVector);
	Vector cel = (Vector)retVec.elementAt(0);
	Vector cdl = (Vector)retVec.elementAt(1);
	chemicalsToDiffuse.addAll(cdl);
	chemicalsToEmit.addAll(cel);
    }
    
    private Vector getSensedChemicalsFromDirection(String dir)
    {
	if(dir == null)
	    return sensedChemicals;
	
	if(dir.equals("X+"))
	    return sensedChemicalsXplus;
	else if(dir.equals("X-"))
	    return sensedChemicalsXminus;
	else if(dir.equals("Y+"))
	    return sensedChemicalsYplus;
	else if(dir.equals("Y-"))
	    return sensedChemicalsYminus;
	else if(dir.equals("Z+"))
	    return sensedChemicalsZplus;
	else if(dir.equals("Z-"))
	    return sensedChemicalsZminus;
	
	return sensedChemicals;
    }
    
    private Vector getSensedChemicalsFromOppositeDirection(String dir)
    {
	if(dir == null)
	    return sensedChemicals;
	
	if(dir.equals("X-"))
	    return sensedChemicalsXplus;
	else if(dir.equals("X+"))
	    return sensedChemicalsXminus;
	else if(dir.equals("Y-"))
	    return sensedChemicalsYplus;
	else if(dir.equals("Y+"))
	    return sensedChemicalsYminus;
	else if(dir.equals("Z-"))
	    return sensedChemicalsZplus;
	else if(dir.equals("Z+"))
	    return sensedChemicalsZminus;
	
	return sensedChemicals;
    }
    
    private boolean checkCondition(ChemicalCondition2 condition, String direction)
    {
	ChemicalConcentration cc, cc1, cc2;
	boolean twoChemicals = false;
	
	cc1 = new ChemicalConcentration(condition.getChemical(), 0.0);
	cc2 = new ChemicalConcentration(new String(""), 0.0);
	if(condition.getBoundType() == 2)
	    {
		twoChemicals = true;
		cc2 = new ChemicalConcentration(condition.getChemical2(), 0.0);
	    }
	
	Vector sc = getSensedChemicalsFromDirection(direction);
	Enumeration enum = sc.elements();
	while(enum.hasMoreElements())
	    {
		cc = (ChemicalConcentration)enum.nextElement();
		if(cc.getChemical().equals(cc1.getChemical()))
		    {
			cc1 = cc;
		    }
		if(twoChemicals && (cc.getChemical().equals(cc2.getChemical())) )
		    cc2 = cc;
	    }
	
	if( twoChemicals )
	    {
		if(cc1.getConcentration() < cc2.getConcentration())
		    return true;
		else
		    return false;
	    }
	else if( condition.getBoundType() == 1)
	    {
		if(cc1.getConcentration() < condition.getBound())
		    {
			return true;
		    }
		else
		    return false;
	    }
	else if( condition.getBoundType() == -1)
	    {
		if(cc1.getConcentration() > condition.getBound())
		    {
			return true;
		    }
		else
		    return false;
	    }
	else if( condition.getBoundType() == 0)
	    {
		if(cc1.getConcentration() == condition.getBound())
		    {
			return true;
		    }
		else
		    return false;
	    }
	
	return false;
    }
    
    public Vector provideService(Vector args)
    {
	if (args == null)
	    return null;
	
	String service = (String)args.elementAt(0);
	
	if(service.equals(new String("Sense Chemicals")))
	    {
		sensedChemicalsXplus = (Vector)args.elementAt(1);
		sensedChemicalsXminus = (Vector)args.elementAt(2);
		sensedChemicalsYplus = (Vector)args.elementAt(3);
		sensedChemicalsYminus = (Vector)args.elementAt(4);
		sensedChemicalsZplus = (Vector)args.elementAt(5);
		sensedChemicalsZminus = (Vector)args.elementAt(6);
		sensedChemicals = (Vector)args.elementAt(7);
		
		return null;
	    }
	else if(service.equals(new String("Emit Chemicals")))
	    {
		Vector argVector = new Vector();
		argVector.add(new String("Diffuse Chemicals"));
		argVector.add(new Integer(getID().intValue()));
		resolveEmissions();
		givenDirection = null;
		argVector.add(chemicalsToEmit);
		argVector.add(chemicalsToDiffuse);
		getSystem().provideService(argVector);		
		return null;
	    }
	else if (service.equals(new String("Store Chemicals")))
	    {
		Vector argVector = new Vector();
		argVector.add(new String("Store Chemicals"));
		argVector.add(new Integer(getID().intValue()));
		argVector.add(chemicalsToStore);
		getSystem().provideService(argVector);
		
		return null;
	    }
	else if (service.equals(new String("Check Condition")))
	    {
		Vector chemConditionsVector = ((ChemicalConditionList2)args.elementAt(1)).getList();
		boolean conditionSatisfied = true;
		
		Enumeration enum = chemConditionsVector.elements();
		while(conditionSatisfied && enum.hasMoreElements())
		    {
			ChemicalCondition2 chemCondition = (ChemicalCondition2)enum.nextElement();
			if (chemCondition.getDirection() != null)
			    {
				if(chemCondition.getDirection().equals("any"))
				    {
					String dir[] = new String[6];
					dir[0] = "X+"; dir[1] = "X-"; dir[2] = "Y+"; dir[3] = "Y-"; dir[4] = "Z+"; dir[5] = "Z-";
					
					String initialDirection = null;
					int start = (int)( Math.random()*6.0 );
					for(int i=0; i<6; i++)
					    {
						if(checkCondition(chemCondition,dir[(start+i)%6]))
						    initialDirection = dir[(start+i)%6];
					    }
					if (initialDirection == null)
					    conditionSatisfied = false;
					else
					    givenDirection = new String(initialDirection);
				    }
				else if(chemCondition.getDirection().equals("opposite"))
				    {
					String direction = null;
					
					if(givenDirection.equals("X+")) direction = "X-";
					else if(givenDirection.equals("X-")) direction = "X+";
					else if(givenDirection.equals("Y+")) direction = "Y-";
					else if(givenDirection.equals("Y-")) direction = "Y+";
					else if(givenDirection.equals("Z+")) direction = "Z-";
					else if(givenDirection.equals("Z-")) direction = "Z+";
					conditionSatisfied = checkCondition(chemCondition, direction);
				    }
				else if(chemCondition.getDirection().equals("same"))
				    {
					conditionSatisfied = checkCondition(chemCondition, givenDirection);
				    }
				else
				    {
					conditionSatisfied = checkCondition(chemCondition, chemCondition.getDirection());
				    }
			    }
			else
			    {
				conditionSatisfied = checkCondition(chemCondition, null);
			    }
		    }

		Vector retVec = new Vector();
		retVec.add(new Boolean(conditionSatisfied));
		if(givenDirection != null)
		    {
			retVec.add(givenDirection);
		    }
		return retVec;
	    }
	else if (service.equals (new String("State Change")))
	    {
		setState((String)args.elementAt(1));
		givenDirection = ((String)args.elementAt(2));
		chemicalsToEmit = (Vector)args.elementAt(3);
		chemicalsToStore = (Vector)args.elementAt(4);
		chemicalsToDiffuse = (Vector)args.elementAt(5);
		
		return null;
	    }
	else if(service.equals(new String("Cell Division")))
	    {
		String state1 = (String)args.elementAt(1);
		String state2 = (String)args.elementAt(2);
		String divisionType = (String)args.elementAt(3);
		givenDirection = ((String)args.elementAt(4));
		chemicalsToEmit = (Vector)args.elementAt(5);
		chemicalsToStore = (Vector)args.elementAt(6);
		chemicalsToDiffuse = (Vector)args.elementAt(7);
		
		Vector argVector = new Vector();
		argVector.add(new String("Cell Division"));
		argVector.add(new Integer(getID().intValue()));
		argVector.add(state1);
		argVector.add(state2);
		argVector.add(divisionType);
		getSystem().provideService(argVector);
		
		return null;
	    }
	else if (service.equals(new String("Cell Death")))
	    {
		givenDirection = ((String)args.elementAt(1));
		chemicalsToEmit = (Vector)args.elementAt(2);
		chemicalsToStore = (Vector)args.elementAt(3);
		chemicalsToDiffuse = (Vector)args.elementAt(4);
		
		Vector argVector = new Vector();
		argVector.add(new String("Cell Death"));
		argVector.add(new Integer(getID().intValue()));
		getSystem().provideService(argVector);
		
		return null;
	    }
	return null;
    }
    
    public String display()
    {
	StringBuffer sb = new StringBuffer();
	String[] A = new String[7];
	A[0] = "X+"; A[1] = "X-"; A[2] = "Y+"; A[3] = "Y-"; A[4] = "Z+"; A[5] = "Z-"; A[6] = "AL";
	
	for(int i=0; i<7; i++)
	    {
		Enumeration enum = getSensedChemicalsFromDirection(A[i]).elements();
		sb.append(" | " + A[i] + " ");
		while(enum.hasMoreElements())
		    {
			ChemicalConcentration cc = (ChemicalConcentration)enum.nextElement();
			sb.append(cc.toString() + ",");
		    }
	    }
	sb.append("\n");
	return sb.toString();
    }
    
    public Color getColor()
    {
	Vector args = new Vector();
	args.add(new String("Get State Color"));
	args.add(new String(getState()));
	Vector retVec = getCellProgram().provideService(args);
	return (Color)retVec.elementAt(0);
    }
    
    static class SimpleCellFactory extends CellFactory
    {
	public Cell create(Integer p_ID, SystemModel p_sys, CellProgram p_cp, String modelOptions, Vector args)
	{
	    Cell cell = new SimpleCell(p_ID, p_sys, p_cp, args);
	    cell.init(args);
	    return cell;
	}
    }
    
    static
    {
	Simulation.cellModels.put(new String("cellmodels.simplecell.SimpleCell"), new SimpleCellFactory());
    }
}
