package core;

import java.util.*;
import core.*;
import util.*;

public abstract class Cell {
  private Integer ID;
  private String[] states;
  private SystemModel sys;
  private CellProgram[] cps;
  private int currentProgram;
  private Random randomNumber;

  public Cell(Integer p_ID, SystemModel p_sys, CellProgram[] p_cps) {
    randomNumber = new Random();
    ID = new Integer(p_ID.intValue());
    sys = p_sys;
    cps = p_cps;
    states = new String[cps.length];
    currentProgram = randomNumber.nextInt(cps.length);
  }

  public Integer getID() {
    return ID;
  }

  public int getCurrentProgram() {
    return currentProgram;
  }

  public String[] getStates() {
    return states;
  }

  public String getExpressedState() {
    return new String(states[currentProgram]);
  }

  public void setExpressedState(String p_state) {
    states[currentProgram] = new String(p_state);
  }

  public void setStates(String[] p_states) {
    if (p_states.length > states.length) {
      System.err.print("p_states is longer than states");
      return;
    }
    for (int program = 0; program < p_states.length; program++) {
      states[program] = p_states[program];
    }
  }

  public SystemModel getSystem() {
    return sys;
  }

  public CellProgram getCellProgram() {
    return cps[currentProgram];
  }

  public CellProgram[] getCellPrograms() {
    return cps;
  }

  public void runCellProgramForOneStep() {
    currentProgram = randomNumber.nextInt(cps.length);
    cps[currentProgram].runCellProgramForOneStep(this);
  }

  public abstract void init(Vector args);

  public abstract Vector provideService(Vector args);

  public abstract Color getColor();
}
