package gui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import gui.SimWinMgr;
import simulator.Simulation;
import simulator.SimulationParameters;

class SimulationWindow extends JInternalFrame implements InternalFrameListener
{
    SimWinMgr simWinMgr;
    Simulation sim;
    Integer windowID;
    
    StatsTableModel statsTable;
    
    /* 3D Display objects */
    TransformGroup objTrans;
    Transform3D trans = new Transform3D();
    JButton buttonRotateX = new JButton("X");
    JButton buttonRotateY = new JButton("Y");
    JButton buttonRotateZ = new JButton("Z");

    JButton buttonStep = new JButton ("Step");
    JButton buttonAdvance = new JButton ("Advance");

    private SimpleUniverse universe = null;
    BranchGroup scene;
    BranchGroup objRoot;
    BranchGroup lightsGroup;
    float angleX = 0.0f, angleY = 0.0f, angleZ = 0.0f;
    
    SimulationWindow (Integer p_windowID, String title, Rectangle bounds, SimWinMgr p_simWinMgr, SimulationParameters params)
    {
	windowID = new Integer (p_windowID.intValue());
	simWinMgr = p_simWinMgr;
	sim = new Simulation (params);
	
	setBounds (bounds);
	setIconifiable (true);
	setResizable (true);
	setMaximizable (true);
	setClosable (true);
	setClosable (true);
	// setTitle (windowID + ". " + title);
	setTitle (sim.getTitle ());
	addInternalFrameListener (this);
	
	/* Create split windows for 3D display, statistics, output and error messages*/
	Container mainContainer = getContentPane();
	mainContainer.setLayout (new BorderLayout ());
	
	JToolBar toolbar = new JToolBar ();

	toolbar.add (buttonStep);
	toolbar.add (buttonAdvance);
	
	toolbar.add (buttonRotateX);
	toolbar.add (buttonRotateY);
	toolbar.add (buttonRotateZ);

	buttonStep.addActionListener (new ButtonItemListener ());
	buttonAdvance.addActionListener (new ButtonItemListener ());

	mainContainer.add (toolbar, BorderLayout.NORTH);

	JSplitPane mainSplitPane, displayStatsSplitPane, outputErrorSplitPane;	
        GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration ();
        Canvas3D p1 = new Canvas3D (config);
	statsTable = new StatsTableModel ();
	JScrollPane p2 = new JScrollPane (new JTable (statsTable));
	JScrollPane p3 = new JScrollPane (new JTextArea());
	JScrollPane p4 = new JScrollPane (new JTextArea());
	
	JTabbedPane jtp = new JTabbedPane ();
	jtp.addTab ("Display", p1);
	jtp.addTab ("Statistics", p2);
	jtp.addTab ("Output", p3);
	jtp.addTab ("Errors", p4);
	mainContainer.add (jtp, BorderLayout.CENTER);
	
	// Create a simple scene and attach it to the virtual universe
	lightsGroup = null;
	objRoot = null;
	universe = new SimpleUniverse(p1);
	universe.getViewingPlatform().setNominalViewingTransform();
	scene = new BranchGroup();
	scene.setCapability( BranchGroup.ALLOW_BOUNDS_READ );
	scene.setCapability( BranchGroup.ALLOW_CHILDREN_WRITE );
	scene.setCapability( BranchGroup.ALLOW_CHILDREN_READ );
	scene.setCapability( BranchGroup.ALLOW_CHILDREN_EXTEND );
	createSceneGraph();
	universe.addBranchGraph(scene);
    }
    
    /* Implementing the InternalFrameListener */
    public void internalFrameClosing(InternalFrameEvent e) {
	simWinMgr.deleteSimulationWindow(windowID);
    }
    
    public void internalFrameClosed(InternalFrameEvent e) {
    }

    public void internalFrameOpened(InternalFrameEvent e) {
    }

    public void internalFrameIconified(InternalFrameEvent e) {
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {
    }

    public void internalFrameActivated(InternalFrameEvent e) {
	simWinMgr.setCurrentSimulationWindow(windowID);
    }
    
    public void internalFrameDeactivated(InternalFrameEvent e) {
    }

    public void createSceneGraph() {
	if (objRoot != null)
	    {
		scene.removeChild (scene.indexOfChild(objRoot));
		objRoot = null;
	    }
	
	// Create the root of the branch graph
	objRoot = new BranchGroup();
	objRoot.setCapability( BranchGroup.ALLOW_DETACH );
	
	// Create the transform group node and initialize it to the
	// identity.  Enable the TRANSFORM_WRITE capability so that
	// our behavior code can modify it at runtime.  Add it to the
	// root of the subgraph.
	objTrans = new TransformGroup();
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objRoot.addChild(objTrans);
	
	TransformGroup objScale = new TransformGroup();
	Transform3D t = new Transform3D();
	t.setScale(0.5);
	objScale.setTransform(t);
	
	// System creates th scene graph
	BranchGroup simGraph = sim.getDisplayGraph();
	objScale.addChild(simGraph);
	objTrans.addChild(objScale);
	
	// create the AWTInteractionBehavior 
	AWTInteractionBehavior awtBehaviorX = new AWTInteractionBehavior(objTrans, 0);
	buttonRotateX.addActionListener(awtBehaviorX);
	AWTInteractionBehavior awtBehaviorY = new AWTInteractionBehavior(objTrans, 1);
	buttonRotateY.addActionListener(awtBehaviorY);
	AWTInteractionBehavior awtBehaviorZ = new AWTInteractionBehavior(objTrans, 2);
	buttonRotateZ.addActionListener(awtBehaviorZ);
	
	// Create a bounds for the background and behaviors
	BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
	awtBehaviorX.setSchedulingBounds(bounds);
	awtBehaviorY.setSchedulingBounds(bounds);
	awtBehaviorZ.setSchedulingBounds(bounds);
	objRoot.addChild(awtBehaviorX);
	objRoot.addChild(awtBehaviorY);
	objRoot.addChild(awtBehaviorZ);
	
	scene.addChild(objRoot);
	
	if(lightsGroup == null)
	    {
		lightsGroup = new BranchGroup();
		// Set up the background
		Color3f bgColor = new Color3f(1.0f, 1.0f, 1.0f);
		Background bg = new Background(bgColor);
		bg.setApplicationBounds(bounds);
		lightsGroup.addChild(bg);
		
		// Set up the global lights
		Color3f lColor1 = new Color3f(0.7f, 0.7f, 0.7f);
		Vector3f lDir1  = new Vector3f(-1.0f, -1.0f, -1.0f);
		Color3f alColor = new Color3f(0.2f, 0.2f, 0.2f);
		
		AmbientLight aLgt = new AmbientLight(alColor);
		aLgt.setInfluencingBounds(bounds);
		DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
		lgt1.setInfluencingBounds(bounds);
		lightsGroup.addChild(aLgt);
		lightsGroup.addChild(lgt1);
		scene.addChild(lightsGroup);
	    }
    }
    
    void nextStep()
    {
	sim.nextStep ();
	createSceneGraph ();
	statsTable.updateStatistics (sim.getStatistics());
	setTitle (sim.getTitle ());
    }
    
    void gotoStep (int stepNumber)
    {
	while (sim.getStep () < stepNumber) {
	    sim.nextStep ();
	    createSceneGraph ();
	}
	
	statsTable.updateStatistics (sim.getStatistics());
	setTitle (sim.getTitle ());
    }
    
    public void destroy()
    {
	universe.removeAllLocales();
    }

    class ButtonItemListener implements ActionListener
    {
	public void actionPerformed (ActionEvent e)
	{
	    JButton button = (JButton)e.getSource();
	    
	    if (button == buttonStep) {
		nextStep ();
	    } else if (button == buttonAdvance) {
		String input = null;
		input = JOptionPane.showInputDialog (getParent(), "Advance how many steps?", "", JOptionPane.QUESTION_MESSAGE);
		if ((input != null) && (input.length() != 0)) {
		    {
			gotoStep ((new Integer(input)).intValue() + sim.getStep ());
		    }
		}
	    }
	}
    }
}
