package gui;

import java.lang.*;
import java.util.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import gui.*;
import core.Simulation;

public class GUI extends JApplet
{
	SimWinMgr simWinMgr;

	public void init()
	{
		String[] args = new String[1];
		Container mainContainer = getContentPane();
		simWinMgr = new SimWinMgr(args);
		mainContainer.add(simWinMgr, BorderLayout.CENTER);

		JMenuBar mainMenu = new MainMenu(simWinMgr);
		setJMenuBar(mainMenu);
	}

	public static void main(String[] args)
	{
		boolean batchMode=false;
		for(int i=0; i<args.length; i++)
		{
			if(args[i].equals("-batch"))
				batchMode = true;
		}

		if(!batchMode)
		{
			JFrame mainWindow = new MainWindow(args);
			mainWindow.setSize(750,750);
			mainWindow.setVisible(true);
		}
		else
		{
			String simFile = new String("");
			String cellProgramFile = new String("");
			String statsFile = new String("");
			String outFile = new String("");
			String errFile = new String("");
			int numSteps = 0;
	
			for(int i=0; i<args.length; i++)
			{
				String param = (String)args[i];
				
				try
				{
					if(param.substring(0,5).equals("-sim:"))
						simFile = param.substring(5);
					else if(param.substring(0,5).equals("-out:"))
						outFile = param.substring(5);
					else if(param.substring(0,5).equals("-err:"))
						errFile = param.substring(5);
					else if(param.substring(0,6).equals("-stat:"))
						statsFile = param.substring(6);
					else if(param.substring(0,6).equals("-batch"))
						;
					else if(param.substring(0,7).equals("-steps:"))
						numSteps = new Integer(param.substring(7)).intValue();
					else if(param.substring(0,9).equals("-program:"))
						cellProgramFile = param.substring(9);
				}
				catch(StringIndexOutOfBoundsException e)
				{
					System.err.println(e.toString());
					System.err.println(param);
				}
			}
			
			if(simFile.equals("") && cellProgramFile.equals(""))
			{
				System.err.println("Usage: java gui.GUI [-batch] -sim:<simFile>"+
									" -program:<cellProgramFile> [-stats:<statFile>]"+
									" [-out:<outFile>] [-err:<errFile>]");
				System.exit(0);
			}

			/*Vector fileNames = new Vector();
			fileNames.add(simFile);
			fileNames.add(cellProgramFile);
			fileNames.add(outFile);
			fileNames.add(statsFile);
			fileNames.add(errFile);*/
	
			BufferedReader cellProgramFileStream;
			BufferedReader simFileStream;
			BufferedWriter outFileStream;
			BufferedWriter statsFileStream;
			BufferedWriter errFileStream;
			Vector ioObjects = new Vector();
	
			try
			{
				simFileStream = new BufferedReader(new FileReader(simFile));
				cellProgramFileStream = new BufferedReader(new FileReader(cellProgramFile));
				if(outFile.equals(""))
				{
					System.out.println("OUT = System.out");
					outFileStream = new BufferedWriter(new PrintWriter(System.out));
				}
				else
				{
					outFileStream = new BufferedWriter(new FileWriter(outFile));
				}
				if(statsFile.equals(""))
				{
					System.out.println("STAT = System.out");
					statsFileStream = new BufferedWriter(new PrintWriter(System.out));
				}
				else
				{
					statsFileStream = new BufferedWriter(new FileWriter(statsFile));
				}
				if(errFile.equals(""))
				{
					System.out.println("ERR = System.err");
					errFileStream = new BufferedWriter(new PrintWriter(System.err));
				}
				else
				{
					errFileStream = new BufferedWriter(new FileWriter(errFile));
				}
				ioObjects.add(simFileStream);
				ioObjects.add(cellProgramFileStream);
				ioObjects.add(outFileStream);
				ioObjects.add(statsFileStream);
				ioObjects.add(errFileStream);

				Simulation theSimulation = new Simulation(ioObjects);
				for(int i=0; i<numSteps; i++)
				{
					theSimulation.nextStep();
				}
				theSimulation.finish();
			}
			catch(FileNotFoundException e)
			{
				System.err.println("Unable to open file: "+e.toString());
				return;
			}
			catch(IOException e)
			{
				System.err.println("Unable to open file: "+e.toString());
				return;
			}
		}
	}
}
