package simulator;

import java.io.*;
import java.util.*;

class SimFileTokenizer extends StreamTokenizer
{
    private boolean eofReached = false;
    
    public SimFileTokenizer(Reader reader)
    {
	super(reader);
	
	resetSyntax();
	wordChars(0,255);
	ordinaryChar(':');
	ordinaryChar('\n');
	whitespaceChars(' ',' ');
	commentChar('#');
	quoteChar('\'');
	quoteChar('\"');
	parseNumbers();
	eolIsSignificant(true);
	slashStarComments(true);
	slashSlashComments(true);
    }
    
    public SimToken nextSimToken() throws IOException
    {
	int token;
	
	boolean start = true;
	boolean isConfig = false;
	boolean done = false;
	boolean isFirstVectorElement = true;
	String configParam = null;
	String configValue = null;
	int instructionNumber = 0;
	Vector instruction = null;
	Vector configOptions = null;
	
	if(eofReached)
	    return new SimToken();
	
	while(!done)
	    {
		switch(token = nextToken())
		    {
		    case StreamTokenizer.TT_EOF:
			{
			    eofReached = true;
			    if(!start)
				{
				    done = true;
				}
			    else
				{
				    return new SimToken();
				}
			}
			
		    case StreamTokenizer.TT_EOL:
			{
			    if(!start)
				{
				    done = true;
				}
			    break;
			}
			
		    case StreamTokenizer.TT_WORD:
			{
			    if(onlyWhiteSpace(sval))
				break;
			    String str = new String(stripWhiteSpace(sval));
			    if(start)
				{
				    start = false;
				    if(str.equals("system") || str.equals("cell") || str.equals("parser") )
					{
					    isConfig = true;
					    configParam = str;
					}
				}
			    else if(isConfig)
				{
				    if(configValue == null)
					{
					    configValue = str;
					}
				    else
					{
					    if(isFirstVectorElement)
						{
						    isFirstVectorElement = false;
						    configOptions = new Vector();
						}
					    configOptions.add(str);
					}
				}
			    else // if instruction
				{
				    if(isFirstVectorElement)
					{
					    isFirstVectorElement = false;
					    instruction = new Vector();
					}
				    instruction.add(str);
				}
			    break;
			}
			
		    case StreamTokenizer.TT_NUMBER:
			{
			    if(start)
				{
				    start = false;
				    isConfig = false;
				    instructionNumber = (int)nval;
				}
			    else if(isConfig)
				{
				    if(isFirstVectorElement)
					{
					    isFirstVectorElement = false;
					    configOptions = new Vector();
					}
				    configOptions.add(new Double(nval));
				}
			    else
				{
				    if(isFirstVectorElement)
					{
					    isFirstVectorElement = false;
					    instruction = new Vector();
					}
				    instruction.add(new Double(nval));
				}
			    break;
			}
			
		    case ':':
			{
			    if(start || isConfig || !isFirstVectorElement)
				return new SimToken(lineno());
			    break;
			}
		    }
	    }
	
	if(isConfig)
	    {
		if(configOptions == null)
		    configOptions = new Vector();
		return new SimToken(configParam, configValue, configOptions);
	    }
	else
	    {
		if(instruction == null)
		    instruction = new Vector();
		return new SimToken(instructionNumber, instruction);
	    }
    }
    
    private static boolean onlyWhiteSpace(String str)
    {
	char ch;
	
	for(int i=0; i<str.length(); i++)
	    {
		ch = str.charAt(i);
		if(!(ch==' ' || ch=='\t' || ch=='\n' || ch=='\r'))
		    {
			return false;
		    }
	    }
	return true;
    }
    
    private String stripWhiteSpace(String str)
    {
	char ch;
	StringBuffer sb = new StringBuffer(str);
	
	for(int i=0; i<sb.length(); )
	    {
		ch = sb.charAt(i);
		if(ch==' ' || ch=='\t' || ch=='\n' || ch=='\r')
		    {
			sb.deleteCharAt(i);
		    }
		else
		    {
			i++;
		    }
	    }
	return new String(sb);
    }
}
